@@ -1,4 +1,4 @@
-/* $Id: acl.c,v 1.111 2013/09/01 04:59:42 manu Exp $ */
+/* $Id: acl.c,v 1.118 2016/11/24 03:59:40 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: acl.c,v 1.111 2013/09/01 04:59:42 manu Exp $");
+__RCSID("$Id: acl.c,v 1.118 2016/11/24 03:59:40 manu Exp $");
 #endif
 #endif
 
@@ -228,14 +228,26 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  AT_STRING, AC_FROM_LIST, AC_EMAIL, EXF_FROM,
 	  acl_print_string, acl_add_string,
 	  acl_free_string, acl_from_cmp },
+	{ AC_RAWFROM, UNIQUE, AS_ANY, "rawfrom", 
+	  AT_STRING, AC_RAWFROM_LIST, AC_EMAIL, EXF_FROM,
+	  acl_print_string, acl_add_string,
+	  acl_free_string, acl_rawfrom_cmp },
 	{ AC_FROM_RE, UNIQUE, AS_ANY, "from_re", 
 	  AT_REGEX, AC_FROM_LIST, AC_REGEX, EXF_FROM,
 	  acl_print_regex, acl_add_regex,
 	  acl_free_regex, acl_from_regexec },
+	{ AC_RAWFROM_RE, UNIQUE, AS_ANY, "rawfrom_re", 
+	  AT_REGEX, AC_RAWFROM_LIST, AC_REGEX, EXF_FROM,
+	  acl_print_regex, acl_add_regex,
+	  acl_free_regex, acl_rawfrom_regexec },
 	{ AC_FROM_LIST, UNIQUE, AS_ANY, "from_list", 
 	  AT_LIST, AC_NONE, AC_NONE, EXF_FROM,
 	  acl_print_list, acl_add_list, 
 	  NULL, acl_list_filter },
+	{ AC_RAWFROM_LIST, UNIQUE, AS_ANY, "rawfrom_list", 
+	  AT_LIST, AC_NONE, AC_NONE, EXF_FROM,
+	  acl_print_list, acl_add_list, 
+	  NULL, acl_list_filter },
 	{ AC_RCPT, MULTIPLE_OK, AS_ANY, "rcpt", 
 	  AT_STRING, AC_RCPT_LIST, AC_EMAIL, EXF_RCPT,
 	  acl_print_string, acl_add_string,
@@ -328,6 +340,10 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  AT_PROP, AC_NONE, AC_PROP_STR, EXF_PROP,
 	  acl_print_prop_string, acl_add_prop_string,
 	  acl_free_prop_string, prop_string_validate },
+	{ AC_PROP_GLOB, MULTIPLE_OK, AS_ANY, "glob_prop", 
+	  AT_PROP, AC_NONE, AC_PROP_GLOB, EXF_PROP,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_glob_validate },
 	{ AC_PROP_PROP, MULTIPLE_OK, AS_ANY, "propprop", 
 	  AT_PROP, AC_NONE, AC_PROP_PROP, EXF_PROP,
 	  acl_print_prop_pop, acl_add_prop_pop,
@@ -809,6 +825,18 @@ acl_from_regexec(ad, stage, ap, priv)
 	return 0;
 }
 
+int
+acl_rawfrom_regexec(ad, stage, ap, priv)
+	acl_data_t *ad;
+	acl_stage_t stage;
+	struct acl_param *ap;
+	struct mlfi_priv *priv;
+{
+	if (myregexec(priv, ad, ap, priv->priv_rawfrom) == 0)
+		return 1;
+	return 0;
+}
+
 int
 acl_auth_regexec(ad, stage, ap, priv)
 	acl_data_t *ad;
@@ -907,12 +935,45 @@ acl_helo_strstr(ad, stage, ap, priv)
 	struct acl_param *ap;
 	struct mlfi_priv *priv;
 {
-	char *helo = ad->string;
+	char *test_helo;
+	char lh_priv[ADDRLEN + 1];
+	char lh_test[ADDRLEN + 1];
+	int i, res;
 
-	printf("-> %s/%s\n", priv->priv_helo, helo);
-	if (strstr(priv->priv_helo, helo) != NULL) 
-		return 1;
-	return 0;
+	res = 0;
+	test_helo = fstring_expand(priv, NULL, ad->string, NULL);
+
+	/* Convert domain-names (ASCII) to same-case for comparison */
+	for (i = 0; i < sizeof(lh_priv) && priv->priv_helo[i]; i++)
+		lh_priv[i] = tolower((int)(priv->priv_helo[i]));
+	lh_priv[i] = '\0';
+
+	for (i = 0; i < sizeof(lh_test) && test_helo[i]; i++)
+		lh_test[i] = tolower((int)(test_helo[i]));
+	lh_test[i] = '\0';
+
+	/*
+	 * Note that for historical reasons (maybe considered a bug,
+	 * but maybe some deployments now rely on this behavior)
+	 * strstr() is used rather than strcmp(), allowing substring
+	 * matches to be treated as good hits.
+	 * Earlier versions did only the strstr(priv_helo,test_helo).
+	 */
+	if ((strstr(lh_priv, lh_test) != NULL) &&
+	    (strstr(lh_test, lh_priv) != NULL)) 
+		res = 1;
+
+	if (conf.c_debug)
+		mg_log(LOG_DEBUG,
+			"acl_helo_strstr: \"%s\"/\"%s\" "
+			"(lowercase of \"%s\"/\"%s\") => %s\n",
+			lh_priv, lh_test,
+			priv->priv_helo, test_helo,
+			res ? "MATCHED" : "MISMATCH");
+
+	free(test_helo);
+
+	return res;
 }
 
 int
@@ -929,6 +990,20 @@ acl_from_cmp(ad, stage, ap, priv)
 	return 0;
 }
 
+int
+acl_rawfrom_cmp(ad, stage, ap, priv)
+	acl_data_t *ad;
+	acl_stage_t stage;
+	struct acl_param *ap;
+	struct mlfi_priv *priv;
+{
+	char *from = ad->string;
+
+	if (emailcmp(priv->priv_rawfrom, from) == 0) 
+		return 1;
+	return 0;
+}
+
 int
 acl_rcpt_cmp(ad, stage, ap, priv)
 	acl_data_t *ad;
@@ -1806,7 +1881,7 @@ void
 acl_free_prop_string(ad)
 	acl_data_t *ad;
 {
-	acl_free_string((void *)&ad->prop->upd_data);
+	acl_free_string((void *)ad->prop->upd_data);
 	free(ad->prop->upd_data);
 	free(ad->prop->upd_name);
 	free(ad->prop);
@@ -1840,7 +1915,7 @@ void
 acl_free_prop_regex(ad)
 	acl_data_t *ad;
 {
-	acl_free_regex((void *)&ad->prop->upd_data);
+	acl_free_regex((void *)ad->prop->upd_data);
 	free(ad->prop->upd_data);
 	free(ad->prop->upd_name);
 	free(ad->prop);
@@ -2236,6 +2311,7 @@ acl_actions(priv, stage, acl, ap, retval)
 			mg_log(LOG_ERR, "strdup failed");
 			exit(EX_OSERR);
 		}
+		priv->priv_sr.sr_addheader_index = ap->ap_addheader_index;
 	}
 	if (ap->ap_addfooter) {
 		priv->priv_sr.sr_addfooter = strdup(ap->ap_addfooter);
@@ -2381,6 +2457,7 @@ acl_filter(stage, ctx, priv)
 		ap.ap_msg = acl->a_msg;
 		ap.ap_report = acl->a_report;
 		ap.ap_addheader = acl->a_addheader;
+		ap.ap_addheader_index = acl->a_addheader_index;
 		ap.ap_addfooter = acl->a_addfooter;
 		ap.ap_subjtag = acl->a_subjtag;
 		ap.ap_maxpeek = acl->a_maxpeek;
@@ -2753,8 +2830,13 @@ acl_entry(entrystr, len, acl)
 		mystrlcat(entrystr, tempstr, len);
 	}
 	if (acl->a_addheader) {
-		snprintf(tempstr, sizeof(tempstr), 
-		    "[addheader \"%s\"] ", acl->a_addheader);
+		if (acl->a_addheader_index >= 0)
+			snprintf(tempstr, sizeof(tempstr), 
+				"[addheader \"%s\",%d] ",
+				acl->a_addheader, acl->a_addheader_index);
+		else
+			snprintf(tempstr, sizeof(tempstr), 
+				"[addheader \"%s\"] ", acl->a_addheader);
 		mystrlcat(entrystr, tempstr, len);
 	}
 	if (acl->a_addfooter) {
@@ -3033,8 +3115,9 @@ acl_add_report(report)
 }
 
 void 
-acl_add_addheader(hdr)
+acl_add_addheader(hdr, index)
 	char *hdr;
+	int index;
 {
 	if (gacl->a_addheader) {
 		mg_log(LOG_ERR,
@@ -3047,9 +3130,11 @@ acl_add_addheader(hdr)
 		    "malloc failed in ACL line %d", conf_line);
 		exit(EX_OSERR);
 	}
+
+	gacl->a_addheader_index = index;
 		
 	if (conf.c_debug || conf.c_acldebug)
-		mg_log(LOG_DEBUG, "load acl addheader \"%s\"", hdr);
+		mg_log(LOG_DEBUG, "load acl addheader \"%s\",%d", hdr, index);
 
 	return;
 }
