@@ -1,4 +1,4 @@
-/* $Id: dnsrbl.h,v 1.10 2007/11/07 00:02:27 manu Exp $ */
+/* $Id: dnsrbl.h,v 1.11 2016/01/03 05:06:32 manu Exp $ */
 
 /*
  * Copyright (c) 2006 Emmanuel Dreyfus
@@ -54,7 +54,7 @@ struct dnsrbl_list {
 void dnsrbl_init(void);
 int dnsrbl_check_source(acl_data_t *, acl_stage_t,
 			struct acl_param *, struct mlfi_priv *);
-void reverse_endian(struct sockaddr *, struct sockaddr *);
+char *iptorevstr(struct sockaddr *, socklen_t, char *, size_t);
 void dnsrbl_source_add(char *, char *, struct sockaddr *, int);
 struct dnsrbl_entry *dnsrbl_byname(char *);
 void dnsrbl_clear(void);
