@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.c,v 1.276 2014/02/07 13:46:46 manu Exp $ */
+/* $Id: milter-greylist.c,v 1.283 2016/11/24 03:59:40 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID  
-__RCSID("$Id: milter-greylist.c,v 1.276 2014/02/07 13:46:46 manu Exp $");
+__RCSID("$Id: milter-greylist.c,v 1.283 2016/11/24 03:59:40 manu Exp $");
 #endif
 #endif
 
@@ -424,9 +424,17 @@ real_connect(ctx, hostname, addr)
 #ifdef USE_GEOIP
 	geoip_set_ccode(priv);
 #endif
+#ifdef HAVE_SPF2
+	priv->priv_spf_header = NULL;
+#endif
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+	priv->priv_spf_result[0] = '\0';
+#endif
 #ifdef USE_DKIM
 	priv->priv_dkim = NULL;
 	priv->priv_dkimstat = DKIM_STAT_OK;
+	priv->priv_dkim_result[0] = '\0';
 #endif
 #ifdef USE_P0F
 	priv->priv_p0f = NULL;
@@ -505,20 +513,11 @@ real_envfrom(ctx, envfrom)
 	priv->priv_spamd_flags = 0;
 #endif
 
-	/* 
-	 * Now let's handle this new message...
+	/*
+	 * Save unchanged from address for rawfrom ACL clauses.
 	 */
-	if ((priv->priv_queueid = smfi_getsymval(ctx, "{i}")) == NULL) {
-#ifndef USE_POSTFIX
-		/* 
-		 * Postfix does not choose a queue file name 
-		 * until after it accepts the first valid RCPT TO 
-		 * command, so don't log the failure 
-		 */
-		mg_log(LOG_DEBUG, "smfi_getsymval failed for {i}");
-#endif
-		priv->priv_queueid = "(unknown id)";
-	}
+	strncpy(priv->priv_rawfrom, *envfrom, ADDRLEN);
+	priv->priv_rawfrom[ADDRLEN] = '\0';
 
 	/*
 	 * Strip spaces from the source address
@@ -585,17 +584,6 @@ real_envfrom(ctx, envfrom)
 	 * Is the sender address SPF-compliant?
 	 */
 	if ((conf.c_nospf == 0) && SPF_CHECK(priv)) {
-		char ipstr[IPADDRSTRLEN];
-
-		if (iptostring(SA(&priv->priv_addr),
-		    priv->priv_addrlen, ipstr, sizeof(ipstr))) {
-
-			mg_log(LOG_INFO,
-			    "%s: Sender IP %s and address %s are "
-			    "SPF-compliant, bypassing greylist", 
-			    priv->priv_queueid, ipstr, *envfrom);
-		}
-
 		priv->priv_sr.sr_elapsed = 0;
 		priv->priv_sr.sr_whitelist = EXF_WHITELIST | EXF_SPF;
 
@@ -628,6 +616,19 @@ real_envrcpt(ctx, envrcpt)
 		mg_log(LOG_ERR, "Internal error: smfi_getpriv() returns NULL");
 		return SMFIS_TEMPFAIL;
 	}
+	if ((priv->priv_queueid = smfi_getsymval(ctx, "{i}")) == NULL) {
+#ifndef USE_POSTFIX
+		/* 
+		 * Postfix does not choose a queue file name 
+		 * until after it accepts the first valid RCPT TO 
+		 * command, unless config option
+		 * 'smtpd_delay_open_until_valid_rcpt = no' set,
+		 * so don't log the failure.
+		 */
+		mg_log(LOG_DEBUG, "smfi_getsymval failed for {i}");
+#endif
+		priv->priv_queueid = "(unknown id)";
+	}
 	if (tarpit_reentry(priv) == SMFIS_TEMPFAIL)
 		return SMFIS_TEMPFAIL;
 
@@ -671,9 +672,13 @@ real_envrcpt(ctx, envrcpt)
 	 * - nospf global parameter is set, or
 	 * - any ACL with spf clause
 	 */
-	if (conf.c_nospf == 0) {
-		if (priv->priv_sr.sr_whitelist & EXF_SPF)
-			goto exit_accept;
+	if ((conf.c_nospf == 0) && (priv->priv_sr.sr_whitelist & EXF_SPF)) {
+		if (priv->priv_rcptcount == 0) // Log only for first recipient
+			mg_log(LOG_INFO,
+				"%s: Sender IP %s and address %s are "
+				"SPF-compliant, bypassing greylist", 
+				priv->priv_queueid, addrstr, priv->priv_from);
+		goto exit_accept;
 	}
 
 	/*
@@ -1247,7 +1252,12 @@ passed:
 		} else {
 			*hdrvalue = '\0';
 			hdrvalue += strlen(sep);
-			smfi_addheader(ctx, hdrname, hdrvalue);
+			if (priv->priv_sr.sr_addheader_index >= 0)
+				smfi_insheader(
+					ctx, priv->priv_sr.sr_addheader_index,
+					hdrname, hdrvalue);
+			else
+				smfi_addheader(ctx, hdrname, hdrvalue);
 		}
 
 		free(hdrname);
@@ -1386,6 +1396,10 @@ passed:
 				ADD_REASON(whystr, "Sender passed SPF test");
 				priv->priv_last_whitelist &= ~EXF_SPF;
 			}
+			if (priv->priv_last_whitelist & EXF_DKIM) {
+				ADD_REASON(whystr, "Sender passed DKIM test");
+				priv->priv_last_whitelist &= ~EXF_DKIM;
+			}
 			if (priv->priv_last_whitelist & EXF_NONIP) {
 #ifdef AF_INET6
 				ADD_REASON(whystr, 
@@ -1486,7 +1500,12 @@ out:
 		} else {
 			*hdrvalue = '\0';
 			hdrvalue += strlen(sep);
-			smfi_addheader(ctx, hdrname, hdrvalue);
+			if (priv->priv_sr.sr_addheader_index >= 0)
+				smfi_insheader(
+					ctx, priv->priv_sr.sr_addheader_index,
+					hdrname, hdrvalue);
+			else
+				smfi_addheader(ctx, hdrname, hdrvalue);
 		}
 
 		free(hdrname);
@@ -1556,6 +1575,10 @@ real_close(ctx)
 		if (priv->priv_p0f != NULL)
 			free(priv->priv_p0f);
 #endif
+#ifdef HAVE_SPF2
+		if (priv->priv_spf_header != NULL)
+			free(priv->priv_spf_header);
+#endif
 #ifdef USE_DKIM
 		dkimcheck_free(priv);
 #endif
@@ -2026,7 +2049,8 @@ main(argc, argv)
  		/* 
  		 * chown the socket so that sendmail does not complain
  		 */
-		if (chown(conf.c_socket, pw->pw_uid, pw->pw_gid) != 0)
+		if ((access(conf.c_socket, F_OK) == 0) &&
+		    (chown(conf.c_socket, pw->pw_uid, pw->pw_gid) != 0))
 			mg_log(LOG_WARNING, "%s: cannot change \"%s\""
 			    " ownership to %s/%s: %s", argv[0], 
 			    conf.c_socket, pw->pw_name, gr->gr_name,
@@ -2444,7 +2468,7 @@ check_drac(dotted_ip)
 {
 	DB *ddb;
 	DBT key, data;
-	char ipkey[16];
+	char ipkey[64];
 	int rc;
 
 	ddb = dbopen(conf.c_dracdb, O_RDONLY | O_SHLOCK, 0666, DB_BTREE, NULL);
@@ -2454,6 +2478,7 @@ check_drac(dotted_ip)
 	}
 
 	key.data = strncpy(ipkey, dotted_ip, sizeof(ipkey));
+	ipkey[sizeof(ipkey) - 1] = '\0';
 	key.size = strlen(ipkey);
 	rc = ddb->get(ddb, &key, &data, 0);
 	ddb->close(ddb);
@@ -2790,7 +2815,7 @@ fstring_expand(priv, rcpt, fstring, cv)
 	size_t outmaxlen = QSTRLEN;
 	char *tmpstr;
 	char *tmpstrp;
-	char *last;
+	char *last = NULL;
 	char *ptok;
 	int fstr_len;	/* format string length, minus the % (eg: %mr -> 2) */
 	int skip_until_brace_close = 0;
@@ -2970,7 +2995,8 @@ fstring_expand(priv, rcpt, fstring, cv)
 			revipstr[len] = '\0';
 
 			/* XXX only works for IPv4? */
-			for (cp = strtok_r(ipstr, ".", &lasts); cp;
+			for (lasts = NULL, cp = strtok_r(ipstr, ".", &lasts);
+			     cp;
 			     cp = strtok_r(NULL, ".", &lasts)) {
 				size_t cplen = strlen(cp);
 				(void)memcpy(revipstr + len - cplen, cp, cplen);
@@ -3119,7 +3145,104 @@ fstring_expand(priv, rcpt, fstring, cv)
 			mystrncat(&outstr, tzstr, &outmaxlen, cv);
 			break;
 		}
-		
+		case 'Q': { /* "Query" results */
+			fstr_len =  2;
+			switch(*(ptok + 1)) {
+			case 's': /* SPF result */
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+				if (priv->priv_spf_result[0])
+					mystrncat(&outstr, 
+					  	priv->priv_spf_result,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			case 'd': /* DKIM result */
+#ifdef USE_DKIM
+				if (priv->priv_dkim_result[0])
+					mystrncat(&outstr, 
+					  	priv->priv_dkim_result,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			case 'A': { /* Authentication-Results header value */
+				char host[ADDRLEN + 1];
+				char output[HDRLEN + 1];
+				char *fqdn;
+				int cx;
+
+				fqdn = smfi_getsymval(priv->priv_ctx, "{j}");
+				if (fqdn == NULL) {
+					mg_log(LOG_DEBUG, 
+					  "smfi_getsymval failed for {j}");
+					gethostname(host, ADDRLEN);
+					fqdn = host;
+				}
+
+				cx = snprintf(output, HDRLEN, "%s", fqdn);
+
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+				if ((cx < HDRLEN) && priv->priv_spf_result[0]) {
+					char tmpfrom[
+						strlen(priv->priv_from) - 1];
+
+					memcpy(tmpfrom,
+						priv->priv_from + 1,
+						sizeof(tmpfrom) - 1);
+					tmpfrom[sizeof(tmpfrom) - 1] = '\0';
+
+					cx += snprintf(output+cx, HDRLEN-cx,
+					      ";\r\n\tspf=%s smtp.mailfrom=%s",
+					      priv->priv_spf_result, tmpfrom);
+				}
+#endif
+#ifdef USE_DKIM
+				if ((cx < HDRLEN) &&
+					priv->priv_dkim_result[0]) {
+					DKIM_SIGINFO *sig=NULL;
+					unsigned const char *domain;
+					cx += snprintf(output+cx, HDRLEN-cx,
+						";\r\n\tdkim=%s",
+						priv->priv_dkim_result);
+
+					if (priv->priv_dkim != NULL)
+						sig = dkim_getsignature(
+							priv->priv_dkim);
+					if ((cx < HDRLEN) && (sig != NULL) &&
+					    ((domain = dkim_sig_getdomain(sig))
+					     != NULL))
+						cx += snprintf(output+cx,
+						HDRLEN-cx, " header.d=%s",
+						domain);
+				}
+#endif
+				if ((cx < HDRLEN) && (cx == strlen(fqdn)))
+					snprintf(output+cx, HDRLEN-cx,
+						"; none");
+				mystrncat(&outstr, output, &outmaxlen, cv);
+				break;
+			}
+			case 'S': /* Received-SPF header value */
+#ifdef HAVE_SPF2
+				if (priv->priv_spf_header != NULL)
+					mystrncat(&outstr, 
+					  	priv->priv_spf_header,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			default:
+				fstr_len = 0;
+				break;
+			}
+			break;
+		}
 		case 'C': {	/* Country code from GeoIP */
 #ifdef USE_GEOIP
 			if (priv->priv_ccode != NULL)
@@ -3611,6 +3734,9 @@ fstring_escape(fstring)
 {
 	char *cp;
 
+	if (fstring == NULL)
+		return NULL;
+
 	for (cp = fstring; *cp != '\0'; cp++) {
 		int slen;
 
