@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.h,v 1.96 2014/02/07 13:46:46 manu Exp $ */
+/* $Id: milter-greylist.h,v 1.99 2016/11/02 17:41:06 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -168,6 +168,7 @@ struct smtp_reply {
 	char *sr_report;
 	char *sr_report_x;
 	char *sr_addheader;
+	int sr_addheader_index;
 	char *sr_addfooter;
 	char *sr_subjtag;
 	sfsistat sr_retcode;
@@ -195,6 +196,7 @@ struct mlfi_priv {
 	char priv_hostname[ADDRLEN + 1];
 	char priv_helo[ADDRLEN + 1];
 	char priv_from[ADDRLEN + 1];
+	char priv_rawfrom[ADDRLEN + 1];
 	LIST_HEAD(, rcpt) priv_rcpt;
 	char priv_cur_rcpt[ADDRLEN + 1];
 	int priv_rcptcount;
@@ -219,9 +221,17 @@ struct mlfi_priv {
 #ifdef USE_DNSRBL
 	LIST_HEAD(, dnsrbl_list) priv_dnsrbl;
 #endif
+#ifdef HAVE_SPF2
+	char *priv_spf_header;
+#endif
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+	char priv_spf_result[10];
+#endif
 #ifdef USE_DKIM
 	DKIM *priv_dkim;
 	DKIM_STAT priv_dkimstat;
+	char priv_dkim_result[10];
 #endif
 #ifdef USE_P0F
 	char *priv_p0f;
@@ -250,9 +260,7 @@ sfsistat mlfi_data(SMFICTX *);
 #endif
 void usage(char *);
 int humanized_atoi(char *);
-#ifndef USE_POSTFIX
 char *local_ipstr(struct mlfi_priv *);
-#endif
 struct in_addr *prefix2mask4(int, struct in_addr *);
 #ifdef AF_INET6
 struct in6_addr *prefix2mask6(int, struct in6_addr *);
