@@ -1,4 +1,4 @@
-/* $Id: pending.h,v 1.43 2010/04/12 12:04:41 manu Exp $ */
+/* $Id: pending.h,v 1.44 2013/01/19 16:01:15 manu Exp $ */
 /* vim: set sw=8 ts=8 sts=8 noet cino=(0: */
 
 /*
@@ -33,7 +33,7 @@
 #define _PENDING_H_
 
 #include "config.h"
-#ifdef HAVE_OLD_QUEUE_H
+#if defined(HAVE_OLD_QUEUE_H) || !defined(HAVE_SYS_QUEUE_H)
 #include "queue.h"
 #else 
 #include <sys/queue.h>
