@@ -1,4 +1,4 @@
-/* $Id: spf.c,v 1.38 2013/10/04 08:01:10 manu Exp $ */
+/* $Id: spf.c,v 1.40 2015/10/30 18:22:30 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2007 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: spf.c,v 1.38 2013/10/04 08:01:10 manu Exp $");
+__RCSID("$Id: spf.c,v 1.40 2015/10/30 18:22:30 manu Exp $");
 #endif
 #endif
 
@@ -96,7 +96,6 @@ spf_check_internal(ad, as, ap, priv)
 	int result = 0;
 	struct timeval tv1, tv2, tv3;
 	enum spf_status status;
-	char *statestr = "";
 
 	if (conf.c_debug)
 		gettimeofday(&tv1, NULL);
@@ -116,45 +115,46 @@ spf_check_internal(ad, as, ap, priv)
 	p->RES = SPF_policy_main(p);
 
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (p->RES == SPF_PASS);
-		statestr = "pass";
+	switch (p->RES) {
+	case SPF_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (p->RES == SPF_H_FAIL);
-		statestr = "fail";
+	case SPF_H_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (p->RES == SPF_S_FAIL);
-		statestr = "softfail";
+	case SPF_S_FAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (p->RES == SPF_NEUTRAL);
-		statestr = "neutral";
+	case SPF_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = (p->RES == SPF_UNKNOWN || p->RES == SPF_UNMECH);
-		statestr = "unknown";
+	case SPF_UNKNOWN:
+	case SPF_UNMECH:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (p->RES == SPF_ERROR);
-		statestr = "error";
+	case SPF_ERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "error");
 		break;
-	case MGSPF_NONE:
-		result = (p->RES == SPF_NONE);
-		statestr = "none";
+	case SPF_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status %d",
-			" spf raw result %d", status, p->RES);
+		mg_log(LOG_ERR, "Internal error: unexpected spf result %d",
+			" spf test code %d", p->RES, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
 	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d (test %s, result %d)",
-			p->RES, statestr, result);
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, status, result);
 
 	SPF_close(p);
 
@@ -211,7 +211,6 @@ spf_check_internal(ad, as, ap, priv)
 	struct timeval tv1, tv2, tv3;
 	size_t len;
 	enum spf_status status;
-	char *statestr = "";
 
 	if (conf.c_debug)
 		gettimeofday(&tv1, NULL);
@@ -271,45 +270,46 @@ spf_check_internal(ad, as, ap, priv)
 #endif
 
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (out.result == SPF_RESULT_PASS);
-		statestr = "pass";
+	switch (out.result) {
+	case SPF_RESULT_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (out.result == SPF_RESULT_FAIL);
-		statestr = "fail";
+	case SPF_RESULT_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (out.result == SPF_RESULT_SOFTFAIL);
-		statestr = "softfail";
+	case SPF_RESULT_SOFTFAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (out.result == SPF_RESULT_NEUTRAL);
-		statestr = "neutral";
+	case SPF_RESULT_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = (out.result == SPF_RESULT_UNKNOWN);
-		statestr = "unknown";
+	case SPF_RESULT_UNKNOWN:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (out.result == SPF_RESULT_ERROR);
-		statestr = "error";
+	case SPF_RESULT_ERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "error");
 		break;
-	case MGSPF_NONE:
-		result = (out.result == SPF_RESULT_NONE);
-		statestr = "none";
+	case SPF_RESULT_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status %d",
-			" spf_alt/spf_2_10 raw result %d", status, out.result);
+		mg_log(LOG_ERR,
+			"Internal error: unexpected spf_alt/spf_2_10 result %d",
+			" spf test code %d", out.result, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
 	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d (test %s, result %d)",
-			out.result, statestr, result);
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, out.result, result);
 
 	SPF_free_output(&out);
 out3:
@@ -340,6 +340,7 @@ spf_check_internal(ad, as, ap, priv)
 	struct mlfi_priv *priv;
 {
 	struct sockaddr *sa = SA(&priv->priv_addr);
+	char *fqdn;
 	char *helo = priv->priv_helo;
 	char *fromp = priv->priv_from;
 	SPF_server_t *spf_server;
@@ -350,7 +351,6 @@ spf_check_internal(ad, as, ap, priv)
 	struct timeval tv1, tv2, tv3;
 	size_t len;
 	enum spf_status status;
-	char *statestr = "";
 
 	if (conf.c_debug)
 		gettimeofday(&tv1, NULL);
@@ -365,6 +365,11 @@ spf_check_internal(ad, as, ap, priv)
 		goto out2;
 	}
 
+	/* Local machine FQDN */
+	fqdn = smfi_getsymval(priv->priv_ctx, "{j}");
+	if (fqdn != NULL && SPF_server_set_rec_dom(spf_server, fqdn) != 0)
+		mg_log(LOG_WARNING, "SPF_server_set_rec_dom failed");
+
 	/*
 	 * Get the IP address
 	 */
@@ -415,46 +420,59 @@ spf_check_internal(ad, as, ap, priv)
 	res = SPF_response_result(spf_response);
 
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (res == SPF_RESULT_PASS);
-		statestr = "pass";
+	switch (res) {
+	case SPF_RESULT_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (res == SPF_RESULT_FAIL);
-		statestr = "fail";
+	case SPF_RESULT_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (res == SPF_RESULT_SOFTFAIL);
-		statestr = "softfail";
+	case SPF_RESULT_SOFTFAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (res == SPF_RESULT_NEUTRAL);
-		statestr = "neutral";
+	case SPF_RESULT_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = 
-		    (res == SPF_RESULT_PERMERROR || res == SPF_RESULT_INVALID);
-		statestr = "unknown";
+	case SPF_RESULT_INVALID:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (res == SPF_RESULT_TEMPERROR);
-		statestr = "error";
+	case SPF_RESULT_PERMERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "permerror");
 		break;
-	case MGSPF_NONE:
-		result = (res == SPF_RESULT_NONE);
-		statestr = "none";
+	case SPF_RESULT_TEMPERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "temperror");
+		break;
+	case SPF_RESULT_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status %d",
-			" spf2 raw result %d", status, res);
+		mg_log(LOG_ERR, "Internal error: unexpected spf2 result %d",
+			" spf test code %d", res, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
 	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d (test %s, result %d)",
-			res, statestr, result);
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, res, result);
+
+	if (priv->priv_spf_header != NULL)
+		free(priv->priv_spf_header);
+
+	
+	if (spf_response->received_spf_value)
+		priv->priv_spf_header =
+		    strdup(spf_response->received_spf_value);
+	else
+		priv->priv_spf_header = NULL;
 
 	SPF_response_free(spf_response);
 out3:
