@@ -1,4 +1,4 @@
-/* $Id: sync.c,v 1.90 2013/03/23 01:45:02 manu Exp $ */
+/* $Id: sync.c,v 1.93 2016/01/31 05:35:38 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2010 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: sync.c,v 1.90 2013/03/23 01:45:02 manu Exp $");
+__RCSID("$Id: sync.c,v 1.93 2016/01/31 05:35:38 manu Exp $");
 #endif
 #endif
 
@@ -48,6 +48,7 @@ __RCSID("$Id: sync.c,v 1.90 2013/03/23 01:45:02 manu Exp $");
 #include <pthread.h>
 #include <syslog.h>
 #include <sysexits.h>
+#include <poll.h>
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -62,6 +63,10 @@ __RCSID("$Id: sync.c,v 1.90 2013/03/23 01:45:02 manu Exp $");
 #include <dmalloc.h> 
 #endif
 
+#ifndef INFTIM
+#define INFTIM -1	/* non standard */
+#endif
+
 struct pending *pending_get(struct sockaddr *, socklen_t, char *, char *,
     time_t, time_t, tuple_t);
 void pending_del(struct sockaddr *, socklen_t, char *, char *, time_t, 
@@ -1317,32 +1322,46 @@ sync_waitdata(fd, stimeout)
 	int fd;
 	time_t stimeout;
 {
-	fd_set fdr, fde;
-	struct timeval timeout;
+	struct pollfd fds;
 	int retval;
 
-	FD_ZERO(&fdr);
-	FD_SET(fd, &fdr);
+	if (stimeout == 0)
+		stimeout = INFTIM;
+	else
+		stimeout = 1000 * stimeout;
 
-	FD_ZERO(&fde);
-	FD_SET(fd, &fde);
+	fds.fd = fd;
+	fds.events = POLLIN;
+	
+	retval = poll(&fds, 1, stimeout);
 
-	timeout.tv_sec = stimeout;
-	timeout.tv_usec = 0;
+	switch (retval) {
+	case 0:
+		mg_log(LOG_ERR, "%s: poll timeout", __func__);
+		break;
 
-	retval = select(fd + 1, &fdr, NULL, &fde, stimeout == 0 ? NULL : &timeout); 
+	case -1:
+		mg_log(LOG_ERR, "%s: poll error: %s",
+		       __func__, strerror(errno));
+		break;
 
-	if (retval == 0) {
-		mg_log(LOG_ERR, "sync_waitdata : select timeout");
-		
-	} else {
-		if (retval < 0) {
-			mg_log(LOG_ERR, "sync_waitdata : error");
-		} else {
-			if (FD_ISSET(fd,&fde)) {
-				mg_log(LOG_ERR, "sync_waitdata : exception fd is set");
-			}
-		}
+	default:
+		if (fds.revents & POLLPRI)
+			mg_log(LOG_ERR, "%s: high priority data available",
+			       __func__);
+
+		if (fds.revents & POLLERR)
+			mg_log(LOG_ERR, "%s: exceptional condition",
+			       __func__);
+
+		if (fds.revents & POLLHUP)
+			mg_log(LOG_ERR, "%s: socket was disconnected",
+			       __func__);
+
+		if (fds.revents & POLLNVAL)
+			mg_log(LOG_ERR, "%s: file descriptor not open",
+			       __func__);
+		break;
 	}
 
 	return retval;
@@ -1368,7 +1387,6 @@ sync_queue(peer, type, pending, autowhite)/* peer list must be read-locked */
 		exit(EX_OSERR);
 	}
 
-	sync->s_peer = peer;
 	sync->s_type = type;
 	sync->s_autowhite = autowhite;
 	sync->s_pending = pending_ref(pending);
@@ -1427,21 +1445,7 @@ sync_sender(dontcare)
 	int done = 0;
 	struct peer *peer;
 	struct sync *sync;
-	pthread_mutex_t mutex;
 	struct timeval tv1, tv2, tv3;
-	int error;
-
-	if ((error = pthread_mutex_init(&mutex, NULL)) != 0) {
-		mg_log(LOG_ERR, "pthread_mutex_init failed: %s", 
-		    strerror(error));
-		exit(EX_OSERR);
-	}
-
-	if ((error = pthread_mutex_lock(&mutex)) != 0) {
-		mg_log(LOG_ERR, "pthread_mutex_lock failed: %s", 
-		    strerror(error));
-		exit(EX_OSERR);
-	}
 
 	for (;;) {
 		pthread_mutex_lock(&sync_dirty_lock);
@@ -1469,7 +1473,7 @@ sync_sender(dontcare)
 
 			while ((sync = sync_queue_peek(peer)) != NULL ) {
 
-				if (sync_send(sync->s_peer, sync->s_type, 
+				if (sync_send(peer, sync->s_type, 
 				    sync->s_pending, sync->s_autowhite) != 0) {
 					if (!sync_queue_poke(peer, sync))
 						sync_free(sync);
@@ -1569,9 +1573,10 @@ select_protocol(peer, s, stream)
 	char *replystr;
 	int replycode;
 	char sep[] = " \n\t\r";
-	char *cookie = NULL;
 
 	for (vers = SYNC_PROTO_CURRENT; vers > 1; vers--) {
+		char *cookie = NULL;
+
 		fprintf(stream, "vers%d\n", vers);
 
 		fflush(stream);
